#include <windows.h>
#include <string>
#include <fstream> 
#include <iostream>

#include "bitmap.h"
#include "bubble_hal.h"

using namespace std;

int getMode();
void processUserImage(BmpParser*, char*);

// constructor
BmpParser::BmpParser() {
}

// destructor
BmpParser::~BmpParser(){
    delete [] onTimeValue;
    delete [] bufferChar;
    delete [] bufferBool;
}

// function for parsing valve on times from a configuration file
int BmpParser::parseConfig(char * configFileName, int maxWidth) {
    configMaxWidth = maxWidth;
    
    // open configuration file
    fileHandle.open(configFileName);
    
    // ensure that configuration file could be opened
    if (fileHandle.fail()) {
        cerr << "Specified configuration file could not be opened. Please check the configuration file name and try again." << endl;
        exit(1);
    }   
    
    // read configuration file data
    onTimeValue = new unsigned short[maxWidth];
    
    for (int i = 0; i < maxWidth; i++) {
        fileHandle >> onTimeValue[i];
        //cout << "Blah " << onTimeValue[i] << endl;
        cTime[i] = onTimeValue[i];
        cout << "Blah " << cTime[i] << endl;
        if (fileHandle.eof()) {
           cerr << "Specified maximum pixel width does not match the information from the configuration file. ";
           cerr << "Please check the configuration file and try again." << endl;
           exit (1);
        }
    }
    
    // check that the configuration file data could be read
    if (fileHandle.bad() || fileHandle.fail()) {
        cerr << "Specified configuration file data could not be read. Please try another configuration file." << endl;
        exit(1);
    }    
    
    // close configuration file
    fileHandle.close();
    return 0;
}
      
// function for parsing bitmap file data
int BmpParser::parseBmp(char * fileName, char * portName, int maxWidth, int pixelBits) {  
    // check that configuration file exists and matched the maximum pixel width
    if (configMaxWidth != maxWidth) {
       cerr << "Specified maximum pixel width does not match the configuration data. Please ensure configuration is correct." << endl;
       exit (1);
    }
    
    // open bitmap file
    fileHandle.open(fileName, ios::in|ios::binary);
    
    // ensure that bitmap file could be opened
    if (fileHandle.fail()) {
        cerr << "Specified input file could not be opened. Please check the input file name and try again." << endl;
        exit(1);
    }

    // read bitmap file header
    fileHandle.read((char *) (&fileHeader.fileType), (sizeof(fileHeader)-2));
    fileHandle.read((char *) (&infoHeader), sizeof(infoHeader));
    getPointer = (int) fileHandle.tellg();
    
    // correct the endianess of the bitmap file header 
    for (int i = 0; i < sizeof(fileHeader)/2; i++) {
        swapEndian((unsigned int *)&fileHeader.fileType+i);
    }

    for (int i = 0; i < sizeof(fileHeader)/2; i++) {
        swapEndian((unsigned int *)&infoHeader+i);
    }  
                 
    // check that the header information indicates the correct file format     
    checkBmp(fileHeader, infoHeader, maxWidth, pixelBits);
 
    // read the bitmap file data
    numPixels   = infoHeader.dataSize*8; 
    bytesPerRow = infoHeader.pixelWidth/8;
    
    bufferChar = new char [infoHeader.dataSize];
    bufferBool = new bool [numPixels];
    
    fileBuffer = fileHandle.rdbuf();
    fileBuffer->pubseekpos(62, ios::in);
    fileBuffer->sgetn((char *) bufferChar, infoHeader.dataSize);
    
    int counter = 0;
    int index   = 0;
    
    // convert the bitmap file data into booleans (individual pixels)
    for (int i = 0; i < infoHeader.dataSize; i++) { 
        //     
        // only convert to boolean if the byte is not a padding
        // (bitmap files will pad each row of pixel data such that it
        // contains a multiple of 4 bytes, e.g. if one row of pixel data
        // can be represented using 1 byte, 3 bytes of padding are added)
        //
        if (counter < (bytesPerRow)%4) {
           bufferBool[index]   = ((short)bufferChar[i] & 0x80) ? 1 : 0;
           bufferBool[index+1] = ((short)bufferChar[i] & 0x40) ? 1 : 0;
           bufferBool[index+2] = ((short)bufferChar[i] & 0x20) ? 1 : 0;
           bufferBool[index+3] = ((short)bufferChar[i] & 0x10) ? 1 : 0;
           bufferBool[index+4] = ((short)bufferChar[i] & 0x08) ? 1 : 0;
           bufferBool[index+5] = ((short)bufferChar[i] & 0x04) ? 1 : 0;
           bufferBool[index+6] = ((short)bufferChar[i] & 0x02) ? 1 : 0;
           bufferBool[index+7] = ((short)bufferChar[i] & 0x01) ? 1 : 0;
           
           index += 8;
        }
        
        counter = (counter == 3) ? 0 : counter+1;
    }
    
    // parse the bitmap file data
    rData.frameType  = 0x01;
    rData.offTime_ms = 800;         
    
    currentX = 0;
    currentY = infoHeader.pixelHeight - 1;
    
    //
    // use colour palette bit to determine 
    // if 0 or 1 represents a black pixel
    //
    isFilled = (infoHeader.colourFill == 0) ? 0 : 1;
 
    //
    // parse the bitmap file data into the according valve on/off times
    // note 1: the bitmap files are stored "upside down", i.e. the first
    // bit of data in a bitmap represents the pixel in the lower left corner
    // of the image
    // note 2: a bit value of 1 indicates a white pixel, 
    //         while a bit value of 0 indicates a black pixel
    //
    for (int i = currentY; i>=0; i--) {
        for (int j = currentX; j<maxWidth; j++) {
            // 
            // if the pixel width is less than the maximum pixel width,
            // zero pad the pixels to the right of the image 
            //
            if (j < infoHeader.pixelWidth) {
               if (bufferBool[i*infoHeader.pixelWidth+j] == isFilled) {
                    //
                    // convert the on time value from the configuration file
                    // from us to 50us divisions
                    //
                    rData.onTime_50us[j] = onTimeValue[j]/50;    
                }
                else {
                    rData.onTime_50us[j] = 0;
                }
            }
            else {
                rData.onTime_50us[j] = 0;
            }
            cout << rData.onTime_50us[j]; // to be removed
        }
        cout << endl; // to be removed
        
        int count = 499;
        while (getMode() != 0) {              
              count++;
              if (count % 500 == 0) {
                        printf("Waiting for Mode 0, current mode: %d\n", getMode());
                        }
              Sleep(10);
              }
        processUserImage(this, portName);                      
        addRow(rData);
        sendData(portName);
    }     
   
    // close the bitmap file
    fileHandle.close();
    return 0;
}

int getMode() {
      int mode;
      ifstream myfile;
      myfile.open ("C:/BubbleDisplay/mode.txt");
      if (!myfile) {
         printf("Error opening mode.txt\n");
         return -1;
      } else {
          myfile >> mode;
          return mode;
       }
      myfile.close();   
}

void processUserImage(BmpParser *p, char * comPort) {
     printf("Processing user image\n");
      ifstream myfile;
      int data[16];
      int num;
      unsigned short * onTimes = p->getOnTimes(); 
      const char * filename = "C:/BubbleDisplay/user.txt";   
      myfile.open (filename);
      if (!myfile) {
         //printf("Error opening mode.txt\n");
         return;
      } else {
          for (int i = 0; i < 16; i++) {
          myfile >> num;
          data[i] = num;
          printf("Got int: %d\n", data[i]);    
              }
          //printf("Sleeping 5000\n");
          //Sleep(5000);
       }
      myfile.close();
      remove (filename); // Delete the file
      
      // Send to serial port
        rowData row;
        row.frameType = 1;
        row.offTime_ms = 800;
        
        // Send 4 rows of ONs followed by 1 row of OFF
        for (int i = 0; i < 24; i++) {
            row.onTime_50us[i] = onTimes[i]/50;
        }
        for (int i = 0; i < 6; i++) {
            while(getMode() != 0) {
                            Sleep(10);
            }
            addRow(row);
            sendData(comPort);
        }
        for (int i = 0; i < 24; i++) {
            row.onTime_50us[i] = 0;
        }       
        while(getMode() != 0) {
                            Sleep(10);
            }
        addRow(row);
        sendData(comPort);        
        
        
                int count = 0;
        for (int i = 0; i < 8; i++) {
            int rIndex = 0;
            for (int j = 0; j < 2; j++) {
                int shifter = 1;
				int temp = data[count];
				printf("Arg %d\n", temp);
                for (int k = 0; k < 12; k++) {
                    if ((temp & shifter) > 0) {                             
                       row.onTime_50us[rIndex] = onTimes[rIndex]/50; 
                       cout << "A" << rIndex << " - " << row.onTime_50us[rIndex] << "  ";  
                       cout << "Writing " << rIndex << endl;     
                    } else {
					   row.onTime_50us[rIndex] = 0;
					   cout << "B";
                       cout << "Writing " << rIndex << endl;					   
				   }
				   //printf("rIndex: %d  onTime: %d\n", rIndex, row.onTime_50us[rIndex]);
				   shifter = shifter * 2;
                    
					rIndex++;	
                }
                count++;
                
            }
			for (int i = 0; i < 24; i++) {
				cout << "R " << row.onTime_50us[i] << endl;
			}
			//system("PAUSE");
           while(getMode() != 0) {
                 Sleep(10);
            }			
			addRow(row);
			sendData(comPort);
            // Send RowData
            // Clear row data
            for (int i = 0; i < 24; i++) {
                row.onTime_50us[i] = 0;
                }
        }
        
                // Send 4 rows of ONs followed by 1 row of OFF
        for (int i = 0; i < 24; i++) {
            row.onTime_50us[i] = onTimes[i]/50;
        }
        for (int i = 0; i < 4; i++) {
            while(getMode() != 0) {
                            Sleep(10);
            }
            addRow(row);
            sendData(comPort);
        }
      
             
}

// function for checking validity of bitmap file header
int BmpParser::checkBmp(bmpFileHeader header, bmpInfoHeader info, 
    int maxWidth, int pixelBits) {     
    //
    // check that the file is a bitmap file, that the maximum pixel width
    // is not exceeded and that it has the correct bits per pixel
    //    
    if (header.fileType != 0x424D) { 
        cerr << "Specified input file is not a bmp (bitmap) file. Please check the input file name and try again." << endl;
        exit (1);
    }
    else if (info.pixelWidth > maxWidth) {
        cerr << "Specified input file pixel width is too wide to fit on the display. Please try another input file." << endl;
        exit (1);
    } 
    else if (info.bitsPerPixel != pixelBits) {
        cerr << "Specified input file has incorrect number of bits per pixel. Please try another input file." << endl;
        exit (1);
    }
    return 0;
}
