#ifndef _bitmap_h_
#define _bitmap_h_

#include <windows.h>
#include <string>
#include <fstream> 
#include <iostream>

#include "bubble_hal.h"

using namespace std;

static unsigned short cTime[24];

// struct declarations to allow usage of structs in other files
typedef struct {
    char  padding1;        // padding so that the first short is byte aligned
    short fileType;        // magic cookie, must be 0x424D (ASCII: BM)
    int   fileSize;        // size of bmp file
    short reserved1;       // application specific, must be 0
    short reserved2;       // application specific, must be 0
    int   dataOffset;      // offset to pixel data
    int   headerLength;    // remaining header length
} bmpFileHeader;

typedef struct {
    int   pixelWidth;      // width in pixels
    int   pixelHeight;     // height in pixels
    short colourPlanes;    // number of colour planes, must be 1 
    short bitsPerPixel;    // number of bits per pixel
    int   compression;     // type of compression, must be 0 (BI_RGB)
    int   dataSize;        // size of pixel data
    int   xResolution;     // horizontal pixels per meter
    int   yResolution;     // vertical pixels per meter
    int   colourUsed;      // number of colours in palette 
    int   colourImportant; // number of important colours
    int   colourFill;      // colour palette bit
} bmpInfoHeader;

// class declaration to allow access to class from other files
class BmpParser {
    private:          
        // private member variable declarations
        bmpFileHeader    fileHeader;     // struct for file header
        bmpInfoHeader    infoHeader;     // struct for information header
        ifstream         fileHandle;     // file handle
        filebuf *        fileBuffer;     // buffer containing file data
        char *           bufferChar;     // temporary storage of pixels (char)
        bool *           bufferBool;     // temporary storage of pixels (bool)
        bool             isFilled;       // colour palette value for black
        int              getPointer;     // pointer to next data byte in file
        int              currentX;       // x-coordinate of pixel being read
        int              currentY;       // y-coordinate of pixel being read
        unsigned int     numPixels;      // total number of pixels
        unsigned int     bytesPerRow;    // number of data bytes per row
        unsigned int     configMaxWidth; // number of configuration entries
        unsigned short * onTimeValue;    // on time for each of the valves
                                         //  (in 50us divisions)
        rowData          rData;          // struct for storing on/off time
             
        // private member functions
        int checkBmp(bmpFileHeader header, bmpInfoHeader info, 
            int maxWidth, int pixelBits);
            
        // function for swapping the byte order for an integer
        inline void swapEndian(unsigned int * input) {
            *input = ((*input & 0xFF000000) >> 8) |
                     ((*input & 0x00FF0000) << 8) | 
                     ((*input & 0x0000FF00) >> 8) |
                     ((*input & 0x000000FF) << 8);            
        }
             
    public:
        // constructer/destructor
        BmpParser();
        ~BmpParser();    
                    
        // public member functions
        int parseConfig(char * configFileName, int maxWidth);
        int parseBmp(char * fileName, char * portName, int maxWidth, int pixelBits);
        unsigned short * getOnTimes() { return onTimeValue; }
};

#endif
