#include <string>

#include "bitmap.h"
#include "bubble_hal.h"

int main(int argc, char *argv[])
{
    char * configFile;
    char * bitmapFile;
    char * comPort;   
    
    BmpParser bitmapParser;
    performanceData pData;
    
    configFile = "config.txt";
    bitmapFile = "images2.bmp";
    comPort    = "COM3";
    
    // parse in command line options
    while ((argc > 1) && (argv[1][0] == '-')) {
        switch (argv[1][1])
        {
            case 'c':
                configFile = &argv[1][2];
                break;
            case 'b':
                bitmapFile = &argv[1][2];
                break;
            case 'p':
                comPort = &argv[1][2];
                break;
            default:
                printf("Argument '%s' not supported\n", argv[1]);
                printf("Supported arguments:\n-c\tConfiguration file\n");
                printf("-b\tBitmap file\n-p\tCOM port\n\n");
                break;
        }
        
        argv++;
        argc--;
    }
    
    // parse configuration file, bitmap file and send row data
    bitmapParser.parseConfig(configFile, 24);
    unsigned short * onTimes = bitmapParser.getOnTimes();    
    for (int i = 0; i < 24; i++) {
        //printf("cTime[%d] = %d\n", i, cTime[i]);
        cout << "onTime[" << i << "] = " << onTimes[i] << endl;
        }

if (argc == 17) {
        rowData row;
        row.frameType = 1;
        row.offTime_ms = 800;
        
        // Send 4 rows of ONs followed by 1 row of OFF
        for (int i = 0; i < 24; i++) {
            row.onTime_50us[i] = onTimes[i]/50;
        }
        for (int i = 0; i < 6; i++) {
            addRow(row);
            sendData(comPort);
        }
        for (int i = 0; i < 24; i++) {
            row.onTime_50us[i] = 0;
        }
        addRow(row);
        sendData(comPort);                
        
        
        int count = 1;
        for (int i = 0; i < 8; i++) {
            int rIndex = 0;
            for (int j = 0; j < 2; j++) {
                int shifter = 1;
				int temp = atoi(argv[count]);
				printf("Arg %d\n", temp);
                for (int k = 0; k < 12; k++) {
                    if ((temp & shifter) > 0) {                             
                       row.onTime_50us[rIndex] = onTimes[rIndex]/50; 
                       cout << "A" << rIndex << " - " << row.onTime_50us[rIndex] << "  ";  
                       cout << "Writing " << rIndex << endl;     
                    } else {
					   row.onTime_50us[rIndex] = 0;
					   cout << "B";
                       cout << "Writing " << rIndex << endl;					   
				   }
				   //printf("rIndex: %d  onTime: %d\n", rIndex, row.onTime_50us[rIndex]);
				   shifter = shifter * 2;
                    
					rIndex++;	
                }
                count++;
                
            }
			for (int i = 0; i < 24; i++) {
				cout << "R " << row.onTime_50us[i] << endl;
			}
			//system("PAUSE");
			addRow(row);
			sendData(comPort);
            // Send RowData
            // Clear row data
            for (int i = 0; i < 24; i++) {
                row.onTime_50us[i] = 0;
                }
        }
    } else {
             while (true) {
                   bitmapParser.parseBmp(bitmapFile, comPort, 24, 1);
             }
    }


    // display performance information
    readPerformanceData(comPort, &pData);
    float rate = (float)pData.num_loops / (float)pData.fifty_micros;
    printf("pData.num_loops = %d    pData.fifty_micros = %d   Rate: %f\n\n", pData.num_loops, pData.fifty_micros, rate);
    
    //system("PAUSE");
    return EXIT_SUCCESS;
}
